/*
 * 12:57:59 31/03/99
 *
 * FindReplace.java - The Jext's find dialog
 * Copyright (C) 1998-1999 Romain Guy
 * powerteam@chez.com
 * www.chez.com/powerteam
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

package com.chez.powerteam.jext;

import java.awt.*;
import gnu.regexp.*;
import javax.swing.*;
import java.awt.event.*;

public class FindReplace extends JDialog implements ActionListener
{
  // Contant declarations

  public static final int SEARCH = 1;
  public static final int REPLACE = 2;

  // Private declarations

  private Jext parent;
  private int selStart, selEnd, type;
  private JCheckBox checkIgnoreCase, saveStates;
  private JTextField fieldSearch = new JTextField(null, 20);
  private JTextField fieldReplace = new JTextField(null, 20);
  private JButton btnFind, btnReplace, btnReplaceAll, btnCancel;

  // This method is used to easily build the GridBagLayout

  void buildConstraints(GridBagConstraints agbc, int agx, int agy, int agw, int agh, int awx, int awy)
  {
    agbc.gridx = agx;
    agbc.gridy = agy;
    agbc.gridwidth = agw;
    agbc.gridheight = agh;
    agbc.weightx = awx;
    agbc.weighty = awy;
    agbc.insets = new Insets(2, 2, 2, 2);
  }

  // FindReplace must have a Jext parent (to call its methods)
  // and a defined type which can be either SEARCH or REPLACE

  public FindReplace(Jext parent, int type, boolean modal)
  {
    super(parent, modal);
    if (type == SEARCH)
      setTitle("Find");
    else if (type == REPLACE)
      setTitle("Replace");
    setResizable(false);
    this.parent = parent;
    this.type = type;

    fieldSearch.setCursor(new Cursor(Cursor.TEXT_CURSOR));
    fieldReplace.setCursor(new Cursor(Cursor.TEXT_CURSOR));

    GridBagLayout gridbag = new GridBagLayout();
    GridBagConstraints constraints = new GridBagConstraints();
    getContentPane().setLayout(gridbag);

    JLabel findLabel = new JLabel("Find:");
    buildConstraints(constraints, 0, 0, 1, 1, 25, 50);
    constraints.anchor = GridBagConstraints.WEST;
    gridbag.setConstraints(findLabel, constraints);
    getContentPane().add(findLabel);

    buildConstraints(constraints, 1, 0, 1, 1, 25, 50);
    constraints.fill = GridBagConstraints.HORIZONTAL;
    constraints.anchor = GridBagConstraints.CENTER;
    gridbag.setConstraints(fieldSearch, constraints);
    getContentPane().add(fieldSearch);

    btnFind = new JButton("Find");
    btnFind.setToolTipText("Find specified string");
    btnFind.setMnemonic('F');
    btnFind.addActionListener(this);
    buildConstraints(constraints, 2, 0, 1, 1, 25, 50);
    constraints.anchor = GridBagConstraints.CENTER;
    gridbag.setConstraints(btnFind, constraints);
    getContentPane().add(btnFind);

    btnCancel = new JButton("Cancel");
    btnCancel.setMnemonic('C');
    btnCancel.addActionListener(this);
    buildConstraints(constraints, 3, 0, 1, 1, 25, 50);
    constraints.anchor = GridBagConstraints.CENTER;
    gridbag.setConstraints(btnCancel, constraints);
    getContentPane().add(btnCancel);

    JLabel replaceLabel = new JLabel("Replace:");
    buildConstraints(constraints, 0, 1, 1, 1, 25, 50);
    constraints.anchor = GridBagConstraints.WEST;
    gridbag.setConstraints(replaceLabel, constraints);
    getContentPane().add(replaceLabel);

    buildConstraints(constraints, 1, 1, 1, 1, 25, 50);
    constraints.fill = GridBagConstraints.HORIZONTAL;
    constraints.anchor = GridBagConstraints.CENTER;
    gridbag.setConstraints(fieldReplace, constraints);
    getContentPane().add(fieldReplace);

    if (type != REPLACE) fieldReplace.setEnabled(false);
    btnReplace = new JButton("Replace");
    btnReplace.setToolTipText("Replace find string");
    btnReplace.setMnemonic('R');
    if (type != REPLACE) btnReplace.setEnabled(false);
    btnReplace.addActionListener(this);
    buildConstraints(constraints, 2, 1, 1, 1, 25, 50);
    constraints.anchor = GridBagConstraints.CENTER;
    gridbag.setConstraints(btnReplace, constraints);
    getContentPane().add(btnReplace);

    btnReplaceAll = new JButton("Replace All");
    btnReplaceAll.setToolTipText("Replace all matches");
    btnReplaceAll.setMnemonic('A');
    if (type != REPLACE) btnReplaceAll.setEnabled(false);
    btnReplaceAll.addActionListener(this);
    buildConstraints(constraints, 3, 1, 1, 1, 25, 50);
    constraints.anchor = GridBagConstraints.CENTER;
    gridbag.setConstraints(btnReplaceAll, constraints);
    getContentPane().add(btnReplaceAll);

    checkIgnoreCase = new JCheckBox("Ignore Case", true);
    buildConstraints(constraints, 0, 2, 1, 1, 50, 50);
    constraints.anchor = GridBagConstraints.WEST;
    gridbag.setConstraints(checkIgnoreCase, constraints);
    getContentPane().add(checkIgnoreCase);

    saveStates = new JCheckBox("Save Values", true);
    buildConstraints(constraints, 1, 2, 1, 1, 50, 50);
    constraints.anchor = GridBagConstraints.WEST;
    gridbag.setConstraints(saveStates, constraints);
    getContentPane().add(saveStates);

    load();

    addWindowListener(new WindowAdapter()
    {
      public void windowClosing(WindowEvent e)
      {
        exit();
      }
    });

    pack();
    setDefaultCloseOperation(DISPOSE_ON_CLOSE);
    Point center = Utilities.getCenterFor(this);
    setLocation(center.x, center.y);
    show();
  }

  // Load some properties

  private void load()
  {
    fieldSearch.setText(Jext.getProperty("find"));
    if (type == REPLACE) fieldReplace.setText(Jext.getProperty("replace"));
    checkIgnoreCase.setSelected("on".equals(Jext.getProperty("ignorecase")));
    saveStates.setSelected("on".equals(Jext.getProperty("savestates")));
  }

  // Save some states

  private void exit()
  {
    if (saveStates.isSelected())
    {
      Jext.setProperty("find", fieldSearch.getText());
      if (type == REPLACE) Jext.setProperty("replace", fieldReplace.getText());
      Jext.setProperty("ignorecase", (checkIgnoreCase.isSelected() ? "on" : "off"));
    } else {
      Jext.setProperty("find", "");
      if (type == REPLACE) Jext.setProperty("replace", "");
      Jext.setProperty("ignorecase", "on");
    }
    Jext.setProperty("savestates", (saveStates.isSelected() ? "on" : "off"));
    dispose();
  }

  // Is in a separate method because replace may call it

  private void find(ActionEvent evt)
  {
    if (!parent.getTextArea().find(fieldSearch.getText(), parent.getTextArea().getCaretPosition(), checkIgnoreCase.isSelected()))
    {
      int response = JOptionPane.showConfirmDialog(null, "Match not found. Restart from beginning ?", "Find", JOptionPane.YES_NO_OPTION, JOptionPane.QUESTION_MESSAGE);
      switch (response)
      {
        case 0:
          parent.getTextArea().setCaretPosition(0);
	  actionPerformed(evt);
          break;
        case 1:
          break;
        default:
          return;
      }
    }
  }

  // Catch the action performed and then look for its source
  // According to the source object we call methods from the
  // parent text area (parent.getTextArea())
  // Some REException have to be caught because of the use of
  // the GNU regexp classes

  public void actionPerformed(ActionEvent evt)
  {
    Object source = evt.getSource();
    selStart = parent.getTextArea().getSelectionStart();
    selEnd = parent.getTextArea().getSelectionEnd();
    if (source == btnCancel)
      exit();
    else if (source == btnFind)
      find(evt);
    else if (source == btnReplace) {
      try
      {
        if (!parent.getTextArea().replaceAll(fieldSearch.getText(), fieldReplace.getText(), selStart, selEnd, checkIgnoreCase.isSelected()))
          Utilities.beep();
	else
	  find(evt);
      } catch (REException ree) { }
    }
    else if (source == btnReplaceAll)
    {
      try
      {
        if (!parent.getTextArea().replaceAll(fieldSearch.getText(), fieldReplace.getText(), 0, parent.getTextArea().getLength(), checkIgnoreCase.isSelected()))
          Utilities.beep();
      } catch (REException ree) { }
    }
  }
}

// End of FindReplace.java
