/*
 * 15:40:24 06/04/99
 *
 * Jext.java - A text editor for Java
 * Copyright (C) 1998-1999 Romain Guy
 * powerteam@chez.com
 * www.chez.com/powerteam
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

package com.chez.powerteam.jext;

import java.io.*;
import java.awt.*;
import java.util.*;
import javax.swing.*;
import java.awt.event.*;
import javax.swing.text.*;
import com.chez.powerteam.jext.menus.*;

public class Jext extends JFrame
{

  // Constants

  public static final String RELEASE = "1.3.3";
  public static final String BUILD = "060499";

  public static String SYSTEM = new String();
  public static final String METAL = "javax.swing.plaf.metal.MetalLookAndFeel";
  public static final String MOTIF = "com.sun.java.swing.plaf.motif.MotifLookAndFeel";

  // Private declarations
  // Have to be static because of the methods below

  private static AutoSave auto;
  private static String usrProps;
  private static Properties props;
  private static SplashScreen splash;
  private static JextTextArea textArea;
  private static JLabel message = new JLabel();
  private static String recent[] = new String[8];
  private static JLabel status = new JLabel(" v" + RELEASE + " - (C)1998-1999 Romain Guy - www.chez.com/powerteam");
  private static final String USER_PROPS = System.getProperty("user.home") + File.separator + ".jext.props";

  // Return the current text area

  public static JextTextArea getTextArea()
  {
    return textArea;
  }

  public static Toolkit getMyToolkit()
  {
    return Toolkit.getDefaultToolkit();
  }

  // Save the menu items properties

  public static void save()
  {
    if (autoSave.getState())
      setProperty("autosave", "on");
    else
      setProperty("autosave", "off");
    if (fontBold.getState())
      setProperty("bold", "on");
    else
      setProperty("bold", "off");
    if (indent.getState())
      setProperty("indent", "on");
    else
      setProperty("indent", "off");
    if (softTab.getState())
      setProperty("softtab", "on");
    else
      setProperty("softtab", "off");
    if (wrapLine.getState())
      setProperty("linewrap", "on");
    else
      setProperty("linewrap", "off");
    if (wrapStyleWord.isSelected())
      setProperty("wrapstyle", "word");
    else
      setProperty("wrapstyle", "char");
    setProperty("font", getTextArea().getFontName());
    Dimension size = getTextArea().getJextParent().getSize();
    setProperty("jext.width", String.valueOf(size.width));
    setProperty("jext.height", String.valueOf(size.height));
    Point location = getTextArea().getJextParent().getLocation();
    setProperty("jext.x", String.valueOf(location.x));
    setProperty("jext.y", String.valueOf(location.y));
    saveProps();
  }

  // Load the menu items properties

  public static void loadProperties()
  {
    if (usrProps != null)
    {
      try
      {
        getTextArea().setTabSize(Integer.parseInt(getProperty("tabsize")));
      } catch (NumberFormatException nf) { }
      try
      {
        getTextArea().setFontSize(Integer.parseInt(getProperty("fontsize")));
      } catch (NumberFormatException nf) { }
      SYSTEM = UIManager.getSystemLookAndFeelClassName();
      indent.setState("on".equals(getProperty("indent")));
      softTab.setState("on".equals(getProperty("softtab")));
      autoSave.setState("on".equals(getProperty("autosave")));
      if (autoSave.getState()) startAutoSave();
      wrapLine.setState("on".equals(getProperty("linewrap")));
      fontBold.setState("bold".equals(getProperty("bold")));
      if (fontBold.getState()) getTextArea().setFontStyle(Font.BOLD);
      wrapStyleWord.setSelected("word".equals(getProperty("wrapstyle")));
      wrapStyleChar.setSelected("char".equals(getProperty("wrapstyle")));
      Color defaultColor = Color.red;
      String hex = Jext.getProperty("caretcolor");
      if (hex != null)
      {
        try
        {
          defaultColor = Color.decode(hex);
        } catch (NumberFormatException nfe) { }
	finally {
	  getTextArea().setCaretColor(defaultColor);
	}
      }
      String fnt = getProperty("font");
      if (fnt != null)
      {
        Enumeration enum = fontGroup.getElements();
        while (enum.hasMoreElements())
        {
          JRadioButtonMenuItem menu = (JRadioButtonMenuItem) enum.nextElement();
          if (fnt.equals(menu.getLabel()))
          {
            getTextArea().setFontName(menu.getLabel());
            menu.setSelected(true);
	    break;
          }
	}
      } else monospaced.setSelected(true);
    }
  }

  // Save the property file

  public static void saveProps()
  {
    if (usrProps != null)
    {
      try
      {
        OutputStream out = new FileOutputStream(usrProps);
        props.store(out, "Jext Properties");
        out.close();
      }
      catch (IOException io)
      {
        io.printStackTrace();
      }
    }
  }

  // Load the property file

  public static void loadProps(InputStream in, String name) throws FileNotFoundException, IOException
  {
    props = new Properties();
    if (in == null)
      in = new FileInputStream(name);
    props.load(in);
    in.close();
  }

  // Save a property

  public static void setProperty(String name, String value)
  {
    props.put(name, value);
  }

  // Return a property from the Jext's property file

  public static String getProperty(String name)
  {
    return props.getProperty(name);
  }

  public static void discard()
  {
    undoUpdate();
    redoUpdate();
  }

  // Start the AutoSave

  public static void startAutoSave()
  {
    if (auto == null) auto = new AutoSave();
  }

  // Stop the AutoSave

  public static void stopAutoSave()
  {
    if (auto != null)
    {
      auto.stop();
      auto = null;
    }
  }

  // Update the status' label

  public static void updateStatus(int off)
  {
    Element map = getTextArea().getDocument().getDefaultRootElement();
    int currLine = map.getElementIndex(off);
    Element lineElement = map.getElement(currLine);
    int start = lineElement.getStartOffset();
    int end = lineElement.getEndOffset();
    int numLines = map.getElementCount();
    status.setText(((off - start) + 1) + ":" + (end - start) + " - " + (currLine + 1) + "/" + numLines + " - " + (((currLine + 1) * 100) / numLines) + "%");
  }

  // Set the GUI look and feel

  public static void setLookAndFeel()
  {
    usrProps = System.getProperty("user.home") + File.separator + ".jext.props";
    if (usrProps != null)
    {
      try
      {
        loadProps(null, USER_PROPS);
      } catch (FileNotFoundException fnf) { }
      catch (IOException e)
      {
        System.err.println("Error while loading user"  + " properties:");
        e.printStackTrace();
      }
      try
      {
        String look = getProperty("lookandfeel");
        if (look == null) look = METAL;
        UIManager.setLookAndFeel(look);
      } catch (Exception e) { }
    }
  }

  // Return the state of the indent check menu item

  public boolean getIndent()
  {
    return indent.getState();
  }

  // Return the state of the softtab check menu item

  public boolean getSoftTab()
  {
    return softTab.getState();
  }

  // Update the undo menu item

  public static void undoUpdate()
  {
    if (getTextArea().getUndo().canUndo())
    {
      undoText.setEnabled(true);
      btnUndo.setEnabled(true);
    } else {
      undoText.setEnabled(false);
      btnUndo.setEnabled(false);
    }
  }

  // Update the redo menu item

  public static void redoUpdate()
  {
    if (getTextArea().getUndo().canRedo())
    {
      redoText.setEnabled(true);
      btnRedo.setEnabled(true);
    } else {
      redoText.setEnabled(false);
      btnRedo.setEnabled(false);
    }
  }

  // Update the paste menu item

  public static void pasteUpdate()
  {
    if (!pasteText.isEnabled())
    {
      pasteText.setEnabled(true);
      btnPaste.setEnabled(true);
    }
  }

  // Update items linked to line wrapping

  public static void wrapUpdate()
  {
    if (wrapLine.getState())
    {
      wrapStyleWord.setEnabled(true);
      wrapStyleChar.setEnabled(true);
    } else {
      wrapStyleWord.setEnabled(false);
      wrapStyleChar.setEnabled(false);
    }
  }

  // If file is new

  public static void setNew()
  {
    if (getTextArea().isEditable())
      message.setText(null);
    else
      message.setText("(read only)");
    updateStatus(0);
  }

  // If file as been changed

  public static void setChanged()
  {
    if (!getTextArea().isDirty())
    {
      getTextArea().setDirty();
      message.setText("(modified)");
    }
  }

  // If file has been saved

  public static void setSaved()
  {
    if (getTextArea().isDirty())
    {
      getTextArea().clean();
      message.setText(null);
    }
  }

  // Quit Jext

  public static void exit()
  {
    if (getTextArea().isDirty() && !getTextArea().isEmpty())
    {
      int response = JOptionPane.showConfirmDialog(null, "Do you want to save your changes ?", "Save", JOptionPane.YES_NO_CANCEL_OPTION, JOptionPane.QUESTION_MESSAGE);
      switch (response)
      {
        case 0:
          getTextArea().saveContent();
	  break;
	case 1:
	  break;
        case 2:
          return;
	default:
	  return;
      }
    }
    stopAutoSave();
    save();
    System.exit(0);
  }

  public JMenu getRecentMenu()
  {
    return menuRecent;
  }

  // Clean the recent menu

  public void removeRecent()
  {
    getRecentMenu().removeAll();
    for (int i = 0; i < 8; i++)
    {
      String prop = getProperty("recent" + i);
      if (prop != null && !prop.equals(""))
        props.remove("recent" + i);
      recent[i] = null;
    }
    JMenuItem nothing = new JMenuItem("No Recent File");
    nothing.setEnabled(false);
    getRecentMenu().add(nothing);
  }

  // Create the recent menu

  public void createRecent(JMenu mnu)
  {
    boolean empty = true;
    mnu.removeAll();
    for (int i = 0; i < 8; i++)
    {
      recent[i] = getProperty("recent" + i);
      if (recent[i] != null && !recent[i].equals(""))
      {
        JMenuItem recentItem = new JMenuItem(Utilities.getShortStringOf(recent[i], 70));
	recentItem.setActionCommand(recent[i]);
        recentItem.addActionListener(new OpenRecent());
        mnu.add(recentItem);
	empty = false;
      }
    }
    if (empty)
    {
      JMenuItem nothing = new JMenuItem("No Recent File");
      nothing.setEnabled(false);
      mnu.add(nothing);
    }
  }

  // Save file in the recent list

  public void saveRecent(String file)
  {
    if (file == null) return;
    for (int i = 7; i > 0; i--)
    {
      recent[i] = recent[i - 1];
      if (recent[i] != null && !recent[i].equals("")) setProperty("recent" + i, recent[i]);
    }
    recent[0] = file;
    setProperty("recent0", file);
    createRecent(menuRecent);
  }

  // Create the insert menu

  public boolean createMenuFromFile(InputStream fileName, JMenu mnu)
  {
    boolean eof = false;
    JMenu menuOld = mnu;
    Vector menus = new Vector();
    Vector items = new Vector();
    JMenu menuParent = new JMenu();
    Insert insertListener = new Insert();

    try
    {
      BufferedReader data = new BufferedReader(new InputStreamReader(fileName));
      while (!eof)
      {
        String mItem = data.readLine();
        if (mItem != null && !mItem.equals(""))
        {
          String header = mItem.substring(0, 1);
          if ((!header.equals("#")) && (!header.equals("@")))
          {
            int separator = mItem.indexOf("::");
            String mItemName = mItem.substring(0, separator).trim();
            if (!mItemName.equals("-"))
            {
              String mItemContent = mItem.substring(separator + 2).trim();
              int star = 0;
              while (star != -1)
              {
                star = mItemContent.indexOf("**");
                if (star != -1)
                  mItemContent = mItemContent.substring(0, star) + "\n" + mItemContent.substring(star + 2);
              }
              items.addElement(new JMenuItem(mItemName));
              mnu.add((JMenuItem) items.lastElement());
              ((JMenuItem) items.lastElement()).setActionCommand(mItemContent);
              ((JMenuItem) items.lastElement()).addActionListener(insertListener);
            } else mnu.addSeparator();
          } else if (header.equals("@")) {
            String level = mItem.substring(1, 2);
            if (!level.equals("0"))
            {
              if (level.equals("1")) mnu = menuOld;
              String menuName = mItem.substring(2).trim();
              menus.addElement(new JMenu(menuName));
              if (level.equals("1"))
	        mnu.add((JMenu) menus.lastElement());
	      else if (level.equals("2"))
	        menuParent.add((JMenu) menus.lastElement());
              mnu = (JMenu) menus.lastElement();
              if (level.equals("1")) menuParent = mnu;
            } else mnu = menuOld;
          }
        } else if (mItem == null)
          eof = true;
      }
    }
    catch (IOException e)
    {
      System.out.println("\nCannot find " + fileName + " !\nInsert menu will be disabled.");
      return false;
    }
    return true;
  }

  // GUI private members

  private static JMenuBar menuBar;
  private static JMenu menuFile, menuEdit, menuHTML, menuAnchor, menuTabs, menuInsert, menuFont, menuOptions, menuJext, menuRecent;
  private static JMenuItem newFile, openFile, clearRecent, saveFile, saveFileAs, exitJext;
  private static JMenuItem undoText, redoText, copyText, cutText, pasteText, selectAll, insertDate, deleteLine, find, replace, gotoLine;
  private static JMenuItem removeSpaces, toTabs, toSpaces, upperCase, lowerCase, expand, setAnchor, gotoAnchor, exchangeAnchor, toAccents, toEntities, wordCount;
  private static JMenuItem about;

  private static JRadioButtonMenuItem serif, sansSerif, dialog, dialogInput, monospaced, wrapStyleWord, wrapStyleChar;
  private static JMenuItem fontSize, tabSize, saveDelay, changeLook, caretColor;
  private static JCheckBoxMenuItem fontBold, wrapLine, indent, autoSave, softTab;

  private static ButtonGroup fontGroup = new ButtonGroup();
  private static ButtonGroup optionGroup = new ButtonGroup();

  private static JButton btnNew, btnOpen, btnSave, btnCopy, btnCut, btnPaste, btnUndo, btnRedo, btnGoto, btnReload, btnFind, btnReplace;

  // Constructor

  Jext()
  {
    super("Jext - Java Text Editor");
    getContentPane().setLayout(new BorderLayout());

    menuBar = new JMenuBar();
    menuFile = new JMenu("File");
    menuFile.setMnemonic('F');
    menuRecent = new JMenu("Open Recent");
    menuEdit = new JMenu("Edit");
    menuEdit.setMnemonic('E');
    menuHTML = new JMenu("HTML");
    menuHTML.setMnemonic('H');
    menuAnchor = new JMenu("Anchor");
    menuAnchor.setMnemonic('A');
    menuTabs = new JMenu("Text");
    menuTabs.setMnemonic('T');
    menuInsert = new JMenu("Insert");
    menuInsert.setMnemonic('I');
    menuFont = new JMenu("Fonts");
    menuFont.setMnemonic('N');
    menuOptions = new JMenu("Options");
    menuOptions.setMnemonic('O');
    menuJext = new JMenu("Jext");
    menuJext.setMnemonic('J');

    menuBar.add(menuFile);
    menuBar.add(menuEdit);
    if (createMenuFromFile(getClass().getResourceAsStream("InsertMenu.txt"), menuInsert)) menuBar.add(menuInsert);
    menuBar.add(menuFont);
    menuBar.add(menuOptions);
    menuBar.add(menuJext);

    newFile = new JMenuItem("New", new ImageIcon(Utilities.getImage("images/menu_new.gif", getClass())));
    newFile.setAccelerator(KeyStroke.getKeyStroke('N', 2));
    openFile = new JMenuItem("Open...", new ImageIcon(Utilities.getImage("images/menu_open.gif", getClass())));
    openFile.setAccelerator(KeyStroke.getKeyStroke('O', 2));
    clearRecent = new JMenuItem("Clear Recent");
    clearRecent.setAccelerator(KeyStroke.getKeyStroke('L', 2));
    saveFile = new JMenuItem("Save", new ImageIcon(Utilities.getImage("images/menu_savefile.gif", getClass())));
    saveFile.setAccelerator(KeyStroke.getKeyStroke('S', 2));
    saveFileAs = new JMenuItem("Save as...");
    exitJext = new JMenuItem("Quit");
    exitJext.setAccelerator(KeyStroke.getKeyStroke('Q', 2));

    undoText = new JMenuItem("Undo", new ImageIcon(Utilities.getImage("images/menu_undo.gif", getClass())));
    undoText.setAccelerator(KeyStroke.getKeyStroke('Z', 2));
    undoText.setEnabled(false);
    redoText = new JMenuItem("Redo", new ImageIcon(Utilities.getImage("images/menu_redo.gif", getClass())));
    redoText.setAccelerator(KeyStroke.getKeyStroke('Y', 2));
    redoText.setEnabled(false);

    copyText = new JMenuItem("Copy", new ImageIcon(Utilities.getImage("images/menu_copy.gif", getClass())));
    copyText.setAccelerator(KeyStroke.getKeyStroke('C', 2));
    cutText = new JMenuItem("Cut", new ImageIcon(Utilities.getImage("images/menu_cut.gif", getClass())));
    cutText.setAccelerator(KeyStroke.getKeyStroke('X', 2));
    pasteText = new JMenuItem("Paste", new ImageIcon(Utilities.getImage("images/menu_paste.gif", getClass())));
    pasteText.setAccelerator(KeyStroke.getKeyStroke('V', 2));

    if (getToolkit().getSystemClipboard().getContents(getTextArea()) == null)
      pasteText.setEnabled(false);

    selectAll = new JMenuItem("Select All");
    selectAll.setAccelerator(KeyStroke.getKeyStroke('A', 2));
    find = new JMenuItem("Find...", new ImageIcon(getClass().getResource("images/menu_find.gif")));
    find.setAccelerator(KeyStroke.getKeyStroke('F', 2));
    replace = new JMenuItem("Replace...");
    replace.setAccelerator(KeyStroke.getKeyStroke('R', 2));

    toSpaces = new JMenuItem("Tabs To Spaces");
    toSpaces.setAccelerator(KeyStroke.getKeyStroke('T', 2 | 8));
    toTabs = new JMenuItem("Spaces To Tabs");
    toTabs.setAccelerator(KeyStroke.getKeyStroke('S', 2 | 8));
    upperCase = new JMenuItem("To Upper Case");
    upperCase.setAccelerator(KeyStroke.getKeyStroke('U', 2 | 8));
    lowerCase = new JMenuItem("To Lower Case");
    lowerCase.setAccelerator(KeyStroke.getKeyStroke('L', 2 | 8));
    removeSpaces = new JMenuItem("Remove End Spaces");
    removeSpaces.setAccelerator(KeyStroke.getKeyStroke('R', 2 | 8));
    wordCount = new JMenuItem("Word Count...");
    wordCount.setAccelerator(KeyStroke.getKeyStroke('W', 2 | 8));
    expand = new JMenuItem("Expand Abreviation");
    expand.setAccelerator(KeyStroke.getKeyStroke('E', 2));

    setAnchor = new JMenuItem("Set Anchor");
    setAnchor.setAccelerator(KeyStroke.getKeyStroke('P', 2 | 8));
    gotoAnchor = new JMenuItem("Goto Anchor");
    gotoAnchor.setAccelerator(KeyStroke.getKeyStroke('G', 2 | 8));
    exchangeAnchor = new JMenuItem("Exchange Caret/Anchor");
    exchangeAnchor.setAccelerator(KeyStroke.getKeyStroke('E', 2 | 8));
    toAccents = new JMenuItem("Entities To Accents");
    toAccents.setAccelerator(KeyStroke.getKeyStroke('H', 2 | 8));
    toEntities = new JMenuItem("Accents To Entities");
    toEntities.setAccelerator(KeyStroke.getKeyStroke('F', 2 | 8));

    deleteLine = new JMenuItem("Delete Line");
    deleteLine.setAccelerator(KeyStroke.getKeyStroke('D', 2));
    gotoLine = new JMenuItem("Goto Line...");
    gotoLine.setAccelerator(KeyStroke.getKeyStroke('G', 2));
    insertDate = new JMenuItem("Date/Time");
    insertDate.setAccelerator(KeyStroke.getKeyStroke('T', 2));

    about = new JMenuItem("About...");
    about.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_F1, 0));

    serif = new JRadioButtonMenuItem("Serif");
    serif.setAccelerator(KeyStroke.getKeyStroke('1', 2));
    fontGroup.add(serif);
    dialog = new JRadioButtonMenuItem("Dialog");
    dialog.setAccelerator(KeyStroke.getKeyStroke('2', 2));
    fontGroup.add(dialog);
    sansSerif = new JRadioButtonMenuItem("SansSerif");
    sansSerif.setAccelerator(KeyStroke.getKeyStroke('3', 2));
    fontGroup.add(sansSerif);
    dialogInput = new JRadioButtonMenuItem("DialogInput");
    dialogInput.setAccelerator(KeyStroke.getKeyStroke('4', 2));
    fontGroup.add(dialogInput);
    monospaced = new JRadioButtonMenuItem("Monospaced", true);
    monospaced.setAccelerator(KeyStroke.getKeyStroke('5', 2));
    fontGroup.add(monospaced);
    fontSize = new JMenuItem("Font Size...");
    fontSize.setAccelerator(KeyStroke.getKeyStroke('I', 2));
    fontBold = new JCheckBoxMenuItem("Bold", false);
    fontBold.setAccelerator(KeyStroke.getKeyStroke('B', 2 | 8));

    wrapLine = new JCheckBoxMenuItem("Line Wrap");
    wrapLine.setAccelerator(KeyStroke.getKeyStroke('W', 2));
    wrapStyleWord = new JRadioButtonMenuItem("Wrap Word", true);
    wrapStyleWord.setAccelerator(KeyStroke.getKeyStroke('M', 2));
    wrapStyleWord.setEnabled(false);
    optionGroup.add(wrapStyleWord);
    wrapStyleChar = new JRadioButtonMenuItem("Wrap Char");
    wrapStyleChar.setAccelerator(KeyStroke.getKeyStroke('H', 2));
    wrapStyleChar.setEnabled(false);
    optionGroup.add(wrapStyleChar);
    tabSize = new JMenuItem("Tab Size...");
    tabSize.setAccelerator(KeyStroke.getKeyStroke('V', 2));
    softTab = new JCheckBoxMenuItem("Soft Tabs");
    softTab.setAccelerator(KeyStroke.getKeyStroke('B', 2));
    indent = new JCheckBoxMenuItem("Auto Indent");
    indent.setAccelerator(KeyStroke.getKeyStroke('I', 2));
    autoSave = new JCheckBoxMenuItem("Auto Save");
    autoSave.setAccelerator(KeyStroke.getKeyStroke('A', 2 | 8));
    saveDelay = new JMenuItem("Save Delay...");
    saveDelay.setAccelerator(KeyStroke.getKeyStroke('D', 2 | 8));
    caretColor = new JMenuItem("Caret Color...");
    caretColor.setAccelerator(KeyStroke.getKeyStroke('C', 2 | 8));
    changeLook = new JMenuItem("Change Look...");
    changeLook.setAccelerator(KeyStroke.getKeyStroke('L', 2));

    newFile.addActionListener(new New());
    openFile.addActionListener(new Open());
    clearRecent.addActionListener(new ClearRecent());
    exitJext.addActionListener(new Quit());
    about.addActionListener(new About());
    copyText.addActionListener(new Copy());
    cutText.addActionListener(new Cut());
    pasteText.addActionListener(new Paste());
    saveFile.addActionListener(new Save());
    saveFileAs.addActionListener(new SaveAs());

    find.addActionListener(new Find());
    replace.addActionListener(new Replace());
    insertDate.addActionListener(new InsertDate());
    deleteLine.addActionListener(new DeleteLine());
    selectAll.addActionListener(new SelectAll());
    gotoLine.addActionListener(new GotoLine());
    toSpaces.addActionListener(new TabsToSpaces());
    toTabs.addActionListener(new SpacesToTabs());
    upperCase.addActionListener(new ToUpperCase());
    lowerCase.addActionListener(new ToLowerCase());
    removeSpaces.addActionListener(new RemoveSpaces());
    wordCount.addActionListener(new WordCount());
    expand.addActionListener(new Expand());
    setAnchor.addActionListener(new SetAnchor());
    gotoAnchor.addActionListener(new GotoAnchor());
    exchangeAnchor.addActionListener(new ExchangeAnchor());
    undoText.addActionListener(new Undo());
    redoText.addActionListener(new Redo());
    wrapLine.addActionListener(new WrapLine());
    wrapStyleWord.addActionListener(new WrapStyleWord());
    wrapStyleChar.addActionListener(new WrapStyleChar());

    dialogInput.addActionListener(new FontChange());
    monospaced.addActionListener(new FontChange());
    dialog.addActionListener(new FontChange());
    serif.addActionListener(new FontChange());
    sansSerif.addActionListener(new FontChange());
    fontSize.addActionListener(new FontSize());
    tabSize.addActionListener(new TabSize());
    fontBold.addActionListener(new Bold());
    saveDelay.addActionListener(new SaveDelay());
    autoSave.addActionListener(new Auto());
    changeLook.addActionListener(new LookChange());
    caretColor.addActionListener(new CaretColor());

    toAccents.addActionListener(new ToAccents());
    toEntities.addActionListener(new ToEntities());

    menuFile.add(newFile);
    menuFile.add(openFile);
    createRecent(menuRecent);
    menuFile.add(menuRecent);
    menuFile.add(clearRecent);
    menuFile.addSeparator();
    menuFile.add(saveFile);
    menuFile.add(saveFileAs);
    menuFile.addSeparator();
    menuFile.add(exitJext);

    menuEdit.add(undoText);
    menuEdit.add(redoText);
    menuEdit.addSeparator();
    menuEdit.add(copyText);
    menuEdit.add(cutText);
    menuEdit.add(pasteText);
    menuEdit.addSeparator();
    menuEdit.add(selectAll);
    menuEdit.addSeparator();
    menuEdit.add(find);
    menuEdit.add(replace);
    menuEdit.addSeparator();
    menuHTML.add(toAccents);
    menuHTML.add(toEntities);
    menuEdit.add(menuHTML);
    menuAnchor.add(setAnchor);
    menuAnchor.add(gotoAnchor);
    menuAnchor.add(exchangeAnchor);
    menuEdit.add(menuAnchor);
    menuTabs.add(toSpaces);
    menuTabs.add(toTabs);
    menuTabs.addSeparator();
    menuTabs.add(upperCase);
    menuTabs.add(lowerCase);
    menuTabs.addSeparator();
    menuTabs.add(removeSpaces);
    menuTabs.add(wordCount);
    menuTabs.add(expand);
    menuEdit.add(menuTabs);
    menuEdit.addSeparator();
    menuEdit.add(gotoLine);
    menuEdit.add(deleteLine);
    menuEdit.add(insertDate);

    menuJext.add(about);

    menuFont.add(serif);
    menuFont.add(dialog);
    menuFont.add(sansSerif);
    menuFont.add(dialogInput);
    menuFont.add(monospaced);
    menuFont.addSeparator();
    menuFont.add(fontSize);
    menuFont.add(fontBold);

    menuOptions.add(wrapLine);
    menuOptions.add(wrapStyleWord);
    menuOptions.add(wrapStyleChar);
    menuOptions.addSeparator();
    menuOptions.add(softTab);
    menuOptions.add(indent);
    menuOptions.add(tabSize);
    menuOptions.addSeparator();
    menuOptions.add(autoSave);
    menuOptions.add(saveDelay);
    menuOptions.addSeparator();
    menuOptions.add(caretColor);
    menuOptions.add(changeLook);

    setJMenuBar(menuBar);

    JToolBar toolBar = new JToolBar();
    btnNew = new JButton(new ImageIcon(getClass().getResource("images/bar_new.gif")));
    btnNew.setToolTipText("New");
    btnNew.addActionListener(new New());
    toolBar.add(btnNew);
    btnOpen = new JButton(new ImageIcon(getClass().getResource("images/bar_open.gif")));
    btnOpen.setToolTipText("Open");
    btnOpen.addActionListener(new Open());
    toolBar.add(btnOpen);
    btnSave = new JButton(new ImageIcon(getClass().getResource("images/bar_save.gif")));
    btnSave.setToolTipText("Save");
    btnSave.addActionListener(new Save());
    toolBar.add(btnSave);
    btnReload = new JButton(new ImageIcon(getClass().getResource("images/bar_reload.gif")));
    btnReload.setToolTipText("Reload");
    btnReload.addActionListener(new Reload());
    toolBar.add(btnReload);
    toolBar.addSeparator();
    btnUndo = new JButton(new ImageIcon(getClass().getResource("images/bar_undo.gif")));
    btnUndo.setToolTipText("Undo");
    btnUndo.addActionListener(new Undo());
    btnUndo.setEnabled(false);
    toolBar.add(btnUndo);
    btnRedo = new JButton(new ImageIcon(getClass().getResource("images/bar_redo.gif")));
    btnRedo.setToolTipText("Redo");
    btnRedo.addActionListener(new Redo());
    btnRedo.setEnabled(false);
    toolBar.add(btnRedo);
    toolBar.addSeparator();
    btnCopy = new JButton(new ImageIcon(getClass().getResource("images/bar_copy.gif")));
    btnCopy.setToolTipText("Copy");
    btnCopy.addActionListener(new Copy());
    toolBar.add(btnCopy);
    btnCut = new JButton(new ImageIcon(getClass().getResource("images/bar_cut.gif")));
    btnCut.setToolTipText("Cut");
    btnCut.addActionListener(new Cut());
    toolBar.add(btnCut);
    btnPaste = new JButton(new ImageIcon(getClass().getResource("images/bar_paste.gif")));
    btnPaste.setToolTipText("Paste");
    btnPaste.addActionListener(new Paste());
    if (getToolkit().getSystemClipboard().getContents(getTextArea()) == null)
      btnPaste.setEnabled(false);
    toolBar.add(btnPaste);
    toolBar.addSeparator();
    btnFind = new JButton(new ImageIcon(getClass().getResource("images/bar_find.gif")));
    btnFind.setToolTipText("Find");
    btnFind.addActionListener(new Find());
    toolBar.add(btnFind);
    btnReplace = new JButton(new ImageIcon(getClass().getResource("images/bar_replace.gif")));
    btnReplace.setToolTipText("Replace");
    btnReplace.addActionListener(new Replace());
    toolBar.add(btnReplace);
    btnGoto = new JButton(new ImageIcon(getClass().getResource("images/bar_goto.gif")));
    btnGoto.setToolTipText("Goto Line");
    btnGoto.addActionListener(new GotoLine());
    toolBar.add(btnGoto);

    setIconImage(Utilities.getImage("images/window_icon.gif", getClass()));
    textArea = new JextTextArea(this);
    JScrollPane scroller = new JScrollPane(textArea, ScrollPaneConstants.VERTICAL_SCROLLBAR_ALWAYS, ScrollPaneConstants.HORIZONTAL_SCROLLBAR_ALWAYS);
    FontMetrics fm = getToolkit().getFontMetrics(textArea.getFont());
    JViewport viewport = scroller.getViewport();
    viewport.setPreferredSize(new Dimension(80 * fm.charWidth('m'), 20 * fm.getHeight()));
    getContentPane().add("Center", scroller);
    JPanel pane = new JPanel();
    pane.setLayout(new BorderLayout());
    pane.add("West", status);
    pane.add("East", message);
    getContentPane().add("South", pane);
    getContentPane().add("North", toolBar);

    addWindowListener(new WindowAdapter()
    {
      public void windowClosing(WindowEvent e)
      {
        exit();
      }
    });

    pack();
    Utilities.loadGeometry(this, "jext");
    if (splash != null) splash.dispose();
    show();
  }

  // Main

  public static void main(String args[])
  {
    setLookAndFeel();
    splash = new SplashScreen();
    Jext gui = new Jext();
    loadProperties();
  }
}

// End of Jext.java
