/*
 * 16:35:56 01/04/99
 *
 * Utilities.java - Some utilities for Jext and its classes
 * Copyright (C) 1999 Romain Guy
 * powerteam@chez.com
 * www.chez.com/powerteam
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

package com.chez.powerteam.jext;

import java.awt.*;
import javax.swing.*;

public class Utilities
{
  // Contains some useful methods

  // Constants

  public static final int OPEN = 0;
  public static final int SAVE = 1;

  // Show an information message wihch text can me modified

  public static void showMessage(String message)
  {
    JOptionPane.showMessageDialog(null, message, "Jext message", JOptionPane.INFORMATION_MESSAGE);
  }

  // Show an error message

  public static void showError(String message)
  {
    JOptionPane.showMessageDialog(null, message, "Error", JOptionPane.ERROR_MESSAGE);
  }

  // Make the sound speaker 'beeping'

  public static void beep()
  {
    Jext.getMyToolkit().beep();
  }

  // Return the screen's dimensions (see getCenterOf)

  public static Dimension getScreenDimension()
  {
    return Jext.getMyToolkit().getScreenSize();
  }

  // Return the point of the screen where the upper left corner of the
  // component should be placed to be centered

  public static Point getCenterFor(Component compo)
  {
    return new Point((getScreenDimension().width - compo.getSize().width) / 2, (getScreenDimension().height - compo.getSize().height) / 2);
  }

  // Return a new image

  public static Image getImage(String picture, Class source)
  {
    return Jext.getMyToolkit().getImage(source.getResource(picture));
  }

  // Show a file chooser dialog
  // Can be called in two modes: OPEN or SAVE

  public static String choosePlace(Component owner, int mode)
  {
    JFileChooser chooser = new JFileChooser(Jext.getProperty("lastdir"));
    if (mode == OPEN)
      chooser.setDialogType(JFileChooser.OPEN_DIALOG);
    else if (mode == SAVE)
      chooser.setDialogType(JFileChooser.SAVE_DIALOG);
    chooser.setFileSelectionMode(JFileChooser.FILES_ONLY);
    chooser.setFileHidingEnabled(true);
    int retVal = chooser.showDialog(owner, null);
    if (retVal == JFileChooser.APPROVE_OPTION)
    {
      Jext.setProperty("lastdir", chooser.getSelectedFile().getParent());
      return chooser.getSelectedFile().getAbsolutePath();
    }
    return null;
  }

  // Create a blank String containing only spaces

  public static String createWhiteSpace(int len)
  {
    StringBuffer buf = new StringBuffer();
    for (int i = 0; i < len; i++)
      buf.append(' ');
    return buf.toString();
  }

  // If longString is too long, it returns it like that:
  // Blabalabalaabal...blabalalabalab

  public static String getShortStringOf(String longString, int maxLength)
  {
    int len = longString.length();
    if (len < maxLength)
      return longString;
    else
      return longString.substring(0, maxLength / 2) + "..." + longString.substring(len - (maxLength / 2));
  }

  // Load the geometry of a window

  public static void loadGeometry(Window win, String name)
  {
    int x, y, width, height;

    try
    {
      width = Integer.parseInt(Jext.getProperty(name + ".width"));
      height = Integer.parseInt(Jext.getProperty(name + ".height"));
    } catch(NumberFormatException nf) {
      Dimension size = win.getSize();
      width = size.width;
      height = size.height;
    }

    try
    {
      x = Integer.parseInt(Jext.getProperty(name + ".x"));
      y = Integer.parseInt(Jext.getProperty(name + ".y"));
    } catch(NumberFormatException nf) {
      Dimension screen = win.getToolkit().getScreenSize();
      x = (screen.width - width) / 2;
      y = (screen.height - height) / 2;
    }

    win.setLocation(x,y);
    win.setSize(width,height);
  }
}

// End of Utilities.java
