/*
 * 18:58:27 25/03/99
 *
 * Indent.java
 * Copyright (C) 1999 Romain Guy
 *
 * This	free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

package com.chez.powerteam.jext.menus;

import javax.swing.text.Element;
import com.chez.powerteam.jext.*;
import java.awt.event.ActionEvent;
import javax.swing.text.BadLocationException;

public class Indent
{
  public static JextTextArea getTextArea()
  {
    return Jext.getTextArea();
  }

  public static boolean indent()
  {
    JextTextArea textArea = getTextArea();
    textArea.beginCompoundEdit();
    int tabSize = textArea.getTabSize();
    Element map = textArea.getDocument().getDefaultRootElement();
    int index = map.getElementIndex(textArea.getCaretPosition());
    if (index == 0) return false;
    Element lineElement = map.getElement(index);
    Element prevLineElement = null;
    int prevStart = 0;
    int prevEnd = 0;
    while (--index >= 0)
    {
      prevLineElement = map.getElement(index);
      prevStart = prevLineElement.getStartOffset();
      prevEnd = prevLineElement.getEndOffset();
      if (prevEnd - prevStart > 1) break;
    }
    if (prevLineElement == null) return false;

    try
    {
      int start = lineElement.getStartOffset();
      int end = lineElement.getEndOffset();
      String line = textArea.getDocument().getText(start, end - start);
      String prevLine = textArea.getDocument().getText(prevStart, prevEnd - prevStart);

      boolean prevLineStart = true;
      int prevLineIndent = 0;
      int prevLineBrackets = 0;
      for (int i = 0; i < prevLine.length(); i++)
      {
        if (!prevLineStart) break;
        switch (prevLine.charAt(i))
        {
          case ' ':
            prevLineIndent++;
            break;
          case '\t':
            prevLineIndent += (tabSize - (prevLineIndent % tabSize));
            break;
          default:
            prevLineStart = false;
            break;
        }
      }

      boolean lineStart = true;
      int lineIndent = 0;
      int lineWidth = 0;

      for (int i = 0; i < line.length(); i++)
      {
        if (!lineStart) break;
        switch (line.charAt(i))
        {
          case ' ':
            lineIndent++;
            lineWidth++;
            break;
          case '\t':
            lineIndent += (tabSize - (lineIndent % tabSize));
            lineWidth++;
            break;
          default:
            lineStart = false;
            break;
        }
      }
      if (lineIndent >= prevLineIndent) return false;

      textArea.getDocument().remove(start,lineWidth);
      textArea.getDocument().insertString(start, Utilities.createWhiteSpace(prevLineIndent), null);
      textArea.endCompoundEdit();
      return true;
    } catch(BadLocationException bl) { }
    return false;
  }
}

// End of Indent.java
